require 'rest_client'

module Biro
  module Bvs
    class Response < BaseResponse
      attr_reader :body

      def initialize(responses)
        @body = { }
        [:score, :debits].each { |r| @body[r] = responses[r].body if responses[r] }
      end

      def returning_score?
        score_body.return_code == "0"
      end

      def returning_debits?
        debits_body.return_code == "0"
      end

      def name
        score_body.response_text_name.strip
      end

      def document
        score_body.response_text_document.to_i
      end

      def scores
        score_body.response_text_result_score.to_i
      end

      def debits?
        debits_body.debits_info == 'S'
      end

      def debits_error_message
        debits_body.error_message.strip unless returning_debits?
      end

      def score_error_message
        score_body.error_message.strip unless returning_score?
      end

      private

      def score_body
        ScoreResponseBuilder.new @body[:score]
      end

      def debits_body
        DebitsResponseBuilder.new @body[:debits]
      end
    end
  end
end
