require 'active_support/core_ext/array'

module Biro
  module Midia100
    class Response < BaseResponse
      def initialize(responses)
        @body = { registration: responses[:registration].body }

        [:rf, :equity, :job, :default].each { |r| @body[r] = responses[r].body if responses[r] }
      end

      def cpf
        text('Cpf')&.gsub(/\D/, '')
      end

      def birthdate
        text('DtNasc')
      end

      def marital_status
        text('EstadoCivil')
      end

      def pis
        text('Pis')
      end

      def mothers_name
        text('NomeMae')
      end

      def income
        text('RendaPresumida')
      end

      def profession
        text('IdProfissao')
      end

      def education
        text('Escolaridade')
      end

      def name
        text('Nome')
      end

      def gender
        text('Sexo')
      end

      def receita_federal
        return nil unless rf

        data = rf['Receita_x0020_Federal_x0020_Pf']

        { name: data.dig('Nome'), status: data.dig('Situacao'), date: data.dig('DataPesquisa'), death: data.dig('Obito') }
      end

      def jobs
        Array.wrap(job&.fetch('Funcionarios', nil)).map do |j|
          {
            cnpj: j['Cnpj']&.gsub(/[\s\.\-\/]/, ''),
            company: j['Empresa']&.strip,
          }
        end
      end

      def defaults
        Array.wrap(default&.fetch('spc', nil)).map do |d|
          {
            bureau: :spc,
            company: d['nomeassociado']&.strip,
            created_at: d['datainclusao'],
            due_at: d['datavencimento'],
            value: d['valor'],
            city: d['nome'],
            state: d['siglauf'],
            phone: { ddd: d['numeroddd'], number: d['numero'] },
            contract: d['contrato']
          }
        end
      end

      def success?
        error.blank?
      end

      def error
        return nil unless registration.keys.first =~ /Erro/

        error = registration.flatten
        "#{error[0]} - #{error[1]['Mensagem']}"
      end

      def addresses
        Array.wrap(registration&.fetch('Enderecos', nil)).map do |e|
          {
            street: e['Logradouro']&.strip,
            number: e['Numero']&.strip,
            complement: e['Complemento']&.strip,
            city: e['Cidade']&.strip,
            state: e['Uf']&.strip,
            zipcode: e['Cep']&.strip
          }
        end
      end

      def emails
        Array.wrap(registration&.fetch('Emails', nil)).map do |e|
          e['EMail']&.strip
        end
      end

      def phones
        landline_phones.to_a + mobile_phones.to_a
      end

      def landline_phones
        Array.wrap(registration&.fetch('Telefones', nil)).map do |t|
          {
            ddd: t['Ddd'],
            number: t['Fone'],
            kind: :landline
          }
        end
      end

      def mobile_phones
        Array.wrap(registration&.fetch('Celulares', nil)).map do |c|
          {
            ddd: c['Ddd'],
            number: c['Fone'],
            kind: :mobile
          }
        end
      end

      private

      def text(field)
        registration.dig('Pf', field)&.strip
      end

      def rf
        result(@body[:rf])
      end

      def default
        result(@body[:default])
      end

      def job
        result(@body[:job])
      end

      def registration
        result(@body[:registration])
      end

      def result(data)
        Nori.new(convert_attributes_to: ->(key, value) { [] }).parse(data[:consultar_response][:consultar_result])['NewDataSet'] unless data.nil?
      end
    end
  end
end
